uw_load_background_rules(_, _) :- % Only load once.
  uw_setting(loaded_background_rules, true, just_fail_if_not_set), !.

uw_load_background_rules(Head, Fold) :-
  uw_set(loaded_background_rules, true),

  cd(background_knowledge),
  ensure_loaded('basic_facts.yap'),
  ensure_loaded('sentence_next_previous.yap'),
  ensure_loaded('word_predicates.yap'),
  ensure_loaded('phrase_predicates.yap'),
  ensure_loaded('id2string.yap'),
  ensure_loaded('domain_specific_word_predicates.yap'),

  ensure_loaded('abstract_properties.yap'),
  ensure_loaded('sentence_properties_quiet.yap'), 
  ensure_loaded('sentence_POS_properties_no_zeroes_quiet.yap'),
  uw_concat(['train', Fold ,'_arg1_differential.yap'],      File1a),
  consult(File1a),
  uw_concat(['train', Fold ,'_arg2_differential.yap'],      File1b),
  consult(File1b),
  uw_concat(['train', Fold ,'_inbetween_differential.yap'], File1c),
  ensure_loaded(File1c),
  uw_concat(['train', Fold ,'_word_odds_counts.yap'],       File2),
  ensure_loaded(File2),
  uw_concat(['train', Fold ,'_word_counts_in_sentences_and_abstracts.yap'], File3),
  ensure_loaded(File3),
  uw_concat(['train', Fold , '_phrases_scored_by_arg1.yap'],            File5a),
  consult(File5a),
  uw_concat(['train', Fold , '_phrases_scored_by_arg2.yap'],            File5b),
  consult(File5b),
  uw_concat(['train', Fold , '_phrases_scored_by_betweenArg1Arg2.yap'], File5c),
  consult(File5c),
  uw_concat(['train', Fold , '_marked_up_words.yap'],                   File6),
  ensure_loaded(File6),

  cd('../dotbfiles'),
  ensure_loaded('sentence_structure.b'),
  ensure_loaded('sentence_properties.b'),
  ensure_loaded('part_of_speech_and_lexical.b'),
  ensure_loaded('arg_frequencies.b'),
  ensure_loaded('between_args_frequencies.b'),
  cd('..'),
  
  % Not needed unless learning, but no harm in loading.
  set_sentence_structure_determinations( Head),
  set_sentence_properties_determinations(Head),
  set_lexical_determinations(            Head),
  set_pos_determinations(                Head),
  set_arg_frequencies(                   Head),
  set_between_arg_frequencies(           Head),
  
  % Necessary to run in vanilla aleph V5, yap 4.5.3
  set(nodes,100),
  !.

uw_load_background_rules(_, _) :- uw_confirm(false, 'uw_load_background_rules failed').  % Make sure no backtracking if this fails (and also dont want two proofs).


% Might want this to be problem-specific.  If so, locate in another file.
assigned_fold(subfold1, test,  fold1).
assigned_fold(subfold2, tune,  fold1).
assigned_fold(subfold3, train, fold1).
assigned_fold(subfold4, train, fold1).
assigned_fold(subfold5, train, fold1).

assigned_fold(subfold1, train, fold2).
assigned_fold(subfold2, test,  fold2).
assigned_fold(subfold3, tune,  fold2).
assigned_fold(subfold4, train, fold2).
assigned_fold(subfold5, train, fold2).

assigned_fold(subfold1, train, fold3).
assigned_fold(subfold2, train, fold3).
assigned_fold(subfold3, test,  fold3).
assigned_fold(subfold4, tune,  fold3).
assigned_fold(subfold5, train, fold3).

assigned_fold(subfold1, train, fold4).
assigned_fold(subfold2, train, fold4).
assigned_fold(subfold3, train, fold4).
assigned_fold(subfold4, test,  fold4).
assigned_fold(subfold5, tune,  fold4).

assigned_fold(subfold1, tune,  fold5).
assigned_fold(subfold2, train, fold5).
assigned_fold(subfold3, train, fold5).
assigned_fold(subfold4, train, fold5).
assigned_fold(subfold5, test,  fold5).


% NOTE: do NOT "reconsult" or "ensure_loader" here since that might erase facts about the train examples.

load_trainset_stuff(Head, Fold) :-
  uw_load_background_rules(Head, Fold),
  uw_setting(use_target_stuff_trainset, UseTargetArgsStuff),
  ((UseTargetArgsStuff, not recorded(uw_have_loaded_train_file, true, _))
    -> uw_concat(['background_knowledge/train', Fold, '_before_between_afters.yap'], File), 
       recorda(uw_have_loaded_train_file, true, _),
       consult(File)
    ;  true),
  load_tuneset_stuff(Head, Fold),
  load_testset_stuff(Head, Fold),
  true.

load_tuneset_stuff(Head, Fold) :- 
  uw_load_background_rules(Head, Fold),
  uw_setting(use_target_stuff_tuneset, UseTargetArgsStuff),
  ((UseTargetArgsStuff, not recorded(uw_have_loaded_tune_file, true, _))
    -> uw_concat(['background_knowledge/tune', Fold, '_before_between_afters.yap'], File),  
       recorda(uw_have_loaded_tune_file, true, _),
       consult(File)
    ; true),
  true.

load_testset_stuff(Head, Fold) :- 
  uw_load_background_rules(Head, Fold),
  uw_setting(use_target_stuff_testset, UseTargetArgsStuff),
  ((UseTargetArgsStuff, not recorded(uw_have_loaded_test_file, true, _))
    -> uw_concat(['background_knowledge/test', Fold, '_before_between_afters.yap'], File),  
       recorda(uw_have_loaded_test_file, true, _),
       consult(File)
    ;  true),
  true.
